package sample;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;

import com.openrules.core.*;

public class TestInProcessOrders {
    public static void main(String[] args) {

        String query = 
                "SELECT c.customerNumber,c.customerName,o.status,o.orderNumber,\r\n"
                + "       p.paymentDate,p.amount \r\n" + "FROM customers c \r\n" + "LEFT JOIN orders o \r\n"
                + "   ON c.customerNumber=o.customerNumber \r\n" + "LEFT JOIN payments p\r\n"
                + "   ON c.customerNumber=p.customerNumber \r\n" + "WHERE \r\n"
                + "    o.status is not NULL AND p.amount is not NULL;";

        System.out.println("Use this SQL query to select not NULL orders");
        System.out.println("\n" + query);

        System.out.println("\nExecuting rules for the selected database records");

        Goal goal = new CustomerSample().createGoal("DefineInProcessOrders");
        goal.put("report", "On");
        Totals totals = new Totals();
        goal.use("Totals", totals);

        Map<String, String> properties = ProjectConfiguration.getProperties();
        String url = properties.get("openrules.sql.url");
        String user = properties.get("openrules.sql.user");
        String password = properties.get("openrules.sql.password");

        try (Connection connection = DriverManager.getConnection(url, user, password);
                PreparedStatement st = connection.prepareStatement(query);
                ResultSet rs = st.executeQuery()) {

            while (rs.next()) {
                Order order = new Order();
                order.setOrderNumber(rs.getInt("o.orderNumber"));
                order.setCustomerNumber(rs.getString("c.customerNumber"));
                order.setCustomerName(rs.getString("c.customerName"));
                order.setStatus(rs.getString("o.status"));
                order.setOrderNumber(rs.getInt("o.orderNumber"));
                order.setPaymentDate(rs.getDate("p.paymentDate"));
                order.setPaymentAmount(rs.getDouble("p.amount"));
                // run goal
                goal.use("Order", order);
                // System.out.println(order.toString());
                goal.execute();

            }

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

        System.out.println(
                "\nFound " + totals.getTotalNumberOfSelectedOrders() + " 'In Process' orders with amount > $80000:");
        if (totals.getTotalNumberOfSelectedOrders() > 0) {
            for (Order order : totals.getSelectedOrders()) {
                System.out.println("Customer Number=" + order.getCustomerNumber() + " Order Number="
                        + order.getOrderNumber() + " Payment Date=" + order.getPaymentDate() + " Payment Amount="
                        + order.getPaymentAmount());
            }
            System.out.println("Total Paid Amount in all selected orders= $" + totals.getTotalPaidAmount());
        }
    }

}