package sample;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;

import com.openrules.core.*;

public class TestShippedOrders {
    public static void main(String[] args) {

        String query = 
                "SELECT c.customerNumber,c.customerName, o.status, o.orderNumber,p.orderPaidAmount\r\n" + 
                "FROM customers c \r\n" + 
                "LEFT JOIN orders o \r\n" + 
                "   ON c.customerNumber=o.customerNumber\r\n" + 
                "LEFT JOIN\r\n" + 
                "  ( SELECT customerNumber, sum(amount) as orderPaidAmount\r\n" + 
                "    FROM payments GROUP BY 1\r\n" + 
                "  ) as p\r\n" + 
                "  ON c.customerNumber=p.customerNumber;";

        System.out.println("Use this SQL query to select orders with paid amounts");
        System.out.println("\n" + query);

        System.out.println("\nExecuting rules for the selected database records");

        Goal goal = new CustomerSample().createGoal("DefineShippedOrders");
        goal.put("report","On");
        Totals totals = new Totals();
        goal.use("Totals", totals);
        
        Map<String,String> properties =  ProjectConfiguration.getProperties();
        String url = properties.get("openrules.sql.url"); 
        String user = properties.get("openrules.sql.user"); 
        String password = properties.get("openrules.sql.password");

        try (Connection connection = DriverManager.getConnection(url, user, password);
                PreparedStatement st = connection.prepareStatement(query);
                ResultSet rs = st.executeQuery()) {

            while (rs.next()) {
                Order order = new Order();
                order.setOrderNumber(rs.getInt("o.orderNumber"));
                order.setCustomerNumber(rs.getString("c.customerNumber"));
                order.setCustomerName(rs.getString("c.customerName"));
                order.setStatus(rs.getString("o.status"));
                order.setOrderNumber(rs.getInt("o.orderNumber"));
                order.setOrderPaidAmount(rs.getDouble("p.orderPaidAmount"));
                // run goal
                goal.use("Order", order);
                goal.execute();

            }

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

        System.out.println(
                "\nFound " + totals.getTotalNumberOfSelectedOrders() + " 'Shipped' orders with paid amount > $150000:");
        if (totals.getTotalNumberOfSelectedOrders() > 0) {
            for (Order order : totals.getSelectedOrders()) {
                System.out.println("Customer Number=" + order.getCustomerNumber() + " Order Number=" + order.getOrderNumber()
                                + " Order Paid Amount=" + order.getOrderPaidAmount());
            }
            System.out.println("Total Paid Amount in all selected orders= $" + totals.getTotalPaidAmount());
        }
    }

}