package sample;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;

import com.openrules.core.*;

public class TestUnpaidAlerts {
   public static void main(String[] args) {
   
      String query = 
              "SELECT\r\n" + 
              "  c.customerNumber,c.customerName,salesRepEmployeeNumber,\r\n" + 
              "  o.totalOrderedAmount,p.totalPaidAmount\r\n" + 
              "FROM customers as c\r\n" + 
              "LEFT JOIN\r\n" + 
              "  ( SELECT customerNumber, sum(amount) as totalPaidAmount\r\n" + 
              "    FROM payments GROUP BY 1\r\n" + 
              "  ) as p\r\n" + 
              "  ON p.customerNumber=c.customerNumber\r\n" + 
              "LEFT JOIN\r\n" + 
              "  ( SELECT customerNumber, sum( quantityOrdered * priceEach) as totalOrderedAmount\r\n" + 
              "    FROM orders\r\n" + 
              "    JOIN orderdetails ods WHERE ods.orderNumber = orders.orderNumber\r\n" + 
              "    GROUP BY 1\r\n" + 
              "  ) as o\r\n" + 
              "  ON o.customerNumber=c.customerNumber;";
      
      System.out.println("Use this SQL query to select unpaid records:");
      System.out.println(query);
      
      Map<String,String> properties =  ProjectConfiguration.getProperties();
      String url = properties.get("openrules.sql.url"); 
      String user = properties.get("openrules.sql.user"); 
      String password = properties.get("openrules.sql.password");
     
      System.out.println("Execute alert rules for the selected unpaid orders");
      Goal goal = new CustomerSample().createGoal("Alerts");
      int n = 0;
      
      UnpaidOrders unpaidOrders = new UnpaidOrders();
      try (Connection connection = DriverManager.getConnection(url, user, password);
              PreparedStatement st = connection.prepareStatement(query);
              ResultSet rs = st.executeQuery()) {
          
          while (rs.next()) {
              unpaidOrders.setCustomerNumber(rs.getString("c.customerNumber"));
              unpaidOrders.setCustomerName(rs.getString("c.customerName"));
              unpaidOrders.setSalesRepEmployeeNumber(rs.getInt("salesRepEmployeeNumber"));
              unpaidOrders.setTotalOrderedAmount(rs.getDouble("totalOrderedAmount"));
              unpaidOrders.setTotalPaidAmount(rs.getDouble("totalPaidAmount"));
              // run goal
              goal.use("UnpaidOrders", unpaidOrders);
              Stats stats = new Stats();
              goal.use("Stats", stats);
              goal.execute();
              if (stats.getAlerts() != null) {
                  
                  for( String alert : stats.getAlerts() ) {
                      System.out.println("Alert #"+ n + ": " + alert );
                      System.out.println("\t" + unpaidOrders);
                      n++;
                  }
              }
          }

      } catch (SQLException e) {
          throw new RuntimeException(e);
      }
	   
   }

} 